Class: Google::Apis::Core::HttpCommand
- Inherits:
-
Object
- Object
- Google::Apis::Core::HttpCommand
- Includes:
- Logging
- Defined in:
- lib/google/apis/core/http_command.rb
Overview
Command for HTTP request/response.
Direct Known Subclasses
Constant Summary collapse
- RETRIABLE_ERRORS =
[Google::Apis::ServerError, Google::Apis::RateLimitError, Google::Apis::TransmissionError]
Instance Attribute Summary collapse
-
#body ⇒ #read
Request body.
-
#connection ⇒ HTTPClient
HTTP Client.
-
#header ⇒ Hash
HTTP headers.
-
#method ⇒ symbol
HTTP method.
-
#options ⇒ Google::Apis::RequestOptions
Request options.
-
#params ⇒ Hash
Path params for URL Template.
-
#query ⇒ Hash
Query params.
-
#url ⇒ String, Addressable::URI
HTTP request URL.
Instance Method Summary collapse
- #allow_form_encoding? ⇒ Boolean
-
#apply_request_options(req_header)
Update the request with any specified options.
-
#authorization_refreshable? ⇒ Boolean
Check if attached credentials can be automatically refreshed.
-
#check_status(status, header = nil, body = nil, message = nil)
Check the response and raise error if needed.
-
#decode_response_body(_content_type, body) ⇒ Object
Process the actual response body.
-
#error(err, rethrow: false) {|nil, err| ... }
Process an error response.
-
#execute(client) {|result, err| ... } ⇒ Object
Execute the command, retrying as necessary.
-
#initialize(method, url, body: nil) ⇒ HttpCommand
constructor
A new instance of HttpCommand.
-
#process_response(status, header, body) ⇒ Object
Check the response and either decode body or raise error.
-
#success(result) {|result, nil| ... } ⇒ Object
Process a success response.
Methods included from Logging
Constructor Details
#initialize(method, url, body: nil) ⇒ HttpCommand
Returns a new instance of HttpCommand.
77 78 79 80 81 82 83 84 85 86 87 |
# File 'lib/google/apis/core/http_command.rb', line 77 def initialize(method, url, body: nil) self. = Google::Apis::RequestOptions.default.dup self.url = url self.url = Addressable::Template.new(url) if url.is_a?(String) self.method = method self.header = Hash.new self.body = body self.query = {} self.params = {} @opencensus_span = nil end |
Instance Attribute Details
#body ⇒ #read
Request body
53 54 55 |
# File 'lib/google/apis/core/http_command.rb', line 53 def body @body end |
#connection ⇒ HTTPClient
HTTP Client
61 62 63 |
# File 'lib/google/apis/core/http_command.rb', line 61 def connection @connection end |
#header ⇒ Hash
HTTP headers
49 50 51 |
# File 'lib/google/apis/core/http_command.rb', line 49 def header @header end |
#method ⇒ symbol
HTTP method
57 58 59 |
# File 'lib/google/apis/core/http_command.rb', line 57 def method @method end |
#options ⇒ Google::Apis::RequestOptions
Request options
41 42 43 |
# File 'lib/google/apis/core/http_command.rb', line 41 def @options end |
#params ⇒ Hash
Path params for URL Template
69 70 71 |
# File 'lib/google/apis/core/http_command.rb', line 69 def params @params end |
#query ⇒ Hash
Query params
65 66 67 |
# File 'lib/google/apis/core/http_command.rb', line 65 def query @query end |
#url ⇒ String, Addressable::URI
HTTP request URL
45 46 47 |
# File 'lib/google/apis/core/http_command.rb', line 45 def url @url end |
Instance Method Details
#allow_form_encoding? ⇒ Boolean
333 334 335 |
# File 'lib/google/apis/core/http_command.rb', line 333 def allow_form_encoding? [:post, :put].include?(method) && body.nil? end |
#apply_request_options(req_header)
This method returns an undefined value.
Update the request with any specified options.
324 325 326 327 328 329 330 331 |
# File 'lib/google/apis/core/http_command.rb', line 324 def (req_header) if ..respond_to?(:apply!) ..apply!(req_header) elsif ..is_a?(String) req_header['Authorization'] = sprintf('Bearer %s', .) end req_header.update(header) end |
#authorization_refreshable? ⇒ Boolean
Check if attached credentials can be automatically refreshed
146 147 148 |
# File 'lib/google/apis/core/http_command.rb', line 146 def ..respond_to?(:apply!) end |
#check_status(status, header = nil, body = nil, message = nil)
This method returns an undefined value.
Check the response and raise error if needed
214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 |
# File 'lib/google/apis/core/http_command.rb', line 214 def check_status(status, header = nil, body = nil, = nil) # TODO: 304 Not Modified depends on context... case status when 200...300 nil when 301, 302, 303, 307 ||= sprintf('Redirect to %s', header['Location']) raise Google::Apis::RedirectError.new(, status_code: status, header: header, body: body) when 401 ||= 'Unauthorized' raise Google::Apis::AuthorizationError.new(, status_code: status, header: header, body: body) when 429 ||= 'Rate limit exceeded' raise Google::Apis::RateLimitError.new(, status_code: status, header: header, body: body) when 304, 400, 402...500 ||= 'Invalid request' raise Google::Apis::ClientError.new(, status_code: status, header: header, body: body) when 500...600 ||= 'Server error' raise Google::Apis::ServerError.new(, status_code: status, header: header, body: body) else logger.warn(sprintf('Encountered unexpected status code %s', status)) ||= 'Unknown error' raise Google::Apis::TransmissionError.new(, status_code: status, header: header, body: body) end end |
#decode_response_body(_content_type, body) ⇒ Object
Process the actual response body. Intended to be overridden by subclasses
248 249 250 |
# File 'lib/google/apis/core/http_command.rb', line 248 def decode_response_body(_content_type, body) body end |
#error(err, rethrow: false) {|nil, err| ... }
This method returns an undefined value.
Process an error response
271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 |
# File 'lib/google/apis/core/http_command.rb', line 271 def error(err, rethrow: false, &block) logger.debug { sprintf('Error - %s', PP.pp(err, '')) } if err.is_a?(HTTPClient::BadResponseError) begin res = err.res raise Google::Apis::TransmissionError.new(err) if res.nil? check_status(res.status.to_i, res.header, res.body) rescue Google::Apis::Error => e err = e end elsif err.is_a?(HTTPClient::TimeoutError) || err.is_a?(SocketError) err = Google::Apis::TransmissionError.new(err) end block.call(nil, err) if block_given? fail err if rethrow || block.nil? end |
#execute(client) {|result, err| ... } ⇒ Object
Execute the command, retrying as necessary
98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 |
# File 'lib/google/apis/core/http_command.rb', line 98 def execute(client) prepare! opencensus_begin_span begin Retriable.retriable tries: .retries + 1, max_elapsed_time: .max_elapsed_time, base_interval: .base_interval, max_interval: .max_interval, multiplier: .multiplier, on: RETRIABLE_ERRORS do |try| # This 2nd level retriable only catches auth errors, and supports 1 retry, which allows # auth to be re-attempted without having to retry all sorts of other failures like # NotFound, etc auth_tries = (try == 1 && ? 2 : 1) Retriable.retriable tries: auth_tries, on: [Google::Apis::AuthorizationError, Signet::AuthorizationError, Signet::RemoteServerError, Signet::UnexpectedStatusError], on_retry: proc { |*| } do execute_once(client).tap do |result| if block_given? yield result, nil end end end end rescue => e if block_given? yield nil, e else raise e end end ensure opencensus_end_span @http_res = nil release! end |
#process_response(status, header, body) ⇒ Object
Check the response and either decode body or raise error
195 196 197 198 |
# File 'lib/google/apis/core/http_command.rb', line 195 def process_response(status, header, body) check_status(status, header, body) decode_response_body(header['Content-Type'].first, body) end |
#success(result) {|result, nil| ... } ⇒ Object
Process a success response
257 258 259 260 261 |
# File 'lib/google/apis/core/http_command.rb', line 257 def success(result, &block) logger.debug { sprintf('Success - %s', safe_object_representation(result)) } block.call(result, nil) if block_given? result end |